@@ -35,6 +35,7 @@
 #ifndef  SPI_KEYMASK_MOD4
   #define  SPI_KEYMASK_MOD4 Mod4Mask
 #endif
+#include "main.h"
 #include "gok-spy.h"
 #include "gok-log.h"
 #include "gok-modifier.h"
@@ -1190,7 +1191,26 @@ AccessibleNode* gok_spy_append_node (AccessibleNode* pNode,
 			}
 		}
 		
-		
+		if (pName == NULL) {
+			/* one last try, we pull chars from description */
+			s = Accessible_getDescription (pAccessible);
+			if (s != NULL) {
+				gint len = strlen (pName);
+				if ( len != 0 )  {
+					if (len > 18) 
+					{
+						gchar *tmp = g_strndup (s, 15);
+						pName = g_strconcat (tmp, "...", NULL);
+						g_free (tmp);
+					}
+					else 
+					{
+						pName = g_strndup (s, len);
+					}
+				}
+				SPI_freeString (s);
+			}
+		}
 		if (pName != NULL) {
 			/* create a new node */
 			gok_spy_add_node (pNode, pAccessible, -1, pName);
@@ -1530,22 +1550,8 @@ void gok_spy_modifier_listener(const AccessibleEvent *event, void *user_data)
 
 	gok_log_enter();
 	gok_log ("event->detail2 = %d", event->detail2);
-
-	
-	for (m = gok_modifier_keymasks; m->name != NULL; m++)
-	{
-		if (event->detail2 & m->keymask)
-		{
-			gok_log ("%s on", m->name);
-			gok_modifier_set_state (m->name, MODIFIER_STATE_ON);
-		}
-		else
-		{
-			gok_log ("%s off", m->name);
-			gok_modifier_set_state (m->name, MODIFIER_STATE_OFF);
-		}
-	}
-
+	gok_keyboard_update_labels ();
+	gok_modifier_update_modifier_keys (gok_main_get_current_keyboard ());
 	gok_log_leave();
 }
 
