@@ -79,6 +79,7 @@
 #define GOK_ERROR_NOSTICKYKEYS		  9
 
 #define GOKMAINKEYBOARDNAME "main"
+#define GOKLOGINKEYBOARDNAME "Keyboard"
 
 #define KEYBOARD_ACCESSIBILITY_STICKY_KEYS_KEY "/desktop/gnome/accessibility/keyboard/stickykeys_enable"
 #define GCONF_ACCESSIBILITY_KEY "/desktop/gnome/interface/accessibility"
@@ -189,7 +190,6 @@ static struct poptOption options[] = {
 		N_("KEYBOARDNAME")
 	},
 	{
-		/* TODO: NOT USED */
 		"login", 
 		'l', 
 		POPT_ARG_NONE, 
@@ -354,6 +354,11 @@ gint gok_main_open(gint argc, gchar *argv[])
 	/* read the user profile data */
 	gok_data_initialize ();
 
+	/* initialize the word prediction engine with gconf values */
+	if (gok_data_get_use_aux_dictionaries ())
+		gok_wordcomplete_set_aux_dictionaries (gok_wordcomplete_get_default (),
+						       gok_data_get_aux_dictionaries ());
+
 	gok_log ("gok_data_initialize has finished");
 
 	/* load the ".rc" files used for the key styles */
@@ -432,9 +437,11 @@ gint gok_main_open(gint argc, gchar *argv[])
 	gok_chunker_initialize();
 	gok_keyboard_initialize();
 	gok_branchbackstack_initialize();
-	gok_main_initialize_wordcomplete ();
 	
-	gok_main_initialize_commandprediction ();
+	if (!bLoginOption) {
+		gok_main_initialize_wordcomplete ();
+		gok_main_initialize_commandprediction ();
+	}
 	
 	/* create the settings dialog */
 	if (gok_settingsdialog_open (bDisplaySettingsDialog) == FALSE)
@@ -443,19 +450,30 @@ gint gok_main_open(gint argc, gchar *argv[])
 		return GOK_ERROR_SETTINGSDIALOG_OPEN;
 	}
 
-	if (mainkeyboardname == NULL)
-	{
-		mainkeyboardname = (gchar*) g_malloc (strlen (GOKMAINKEYBOARDNAME) + 1);
-		strcpy (mainkeyboardname, (gchar *) GOKMAINKEYBOARDNAME);
-	}
-	
 
-	gok_main_check_accessibility();
+	/* a specified keyboard takes precedance, even if the login flag is specifed
+	   we use the login keyboard, otherwise the regular main keyboard */
+	if (mainkeyboardname == NULL) {
+		if (bLoginOption) {
+			mainkeyboardname = (gchar*) 
+				g_malloc (strlen (GOKLOGINKEYBOARDNAME) + 1);
+			strcpy (mainkeyboardname, (gchar *) GOKLOGINKEYBOARDNAME);
+		}
+		else {
+			mainkeyboardname = (gchar*) 
+				g_malloc (strlen (GOKMAINKEYBOARDNAME) + 1);
+			strcpy (mainkeyboardname, (gchar *) GOKMAINKEYBOARDNAME);
+		}
+	}
+
+	if (!bLoginOption) {
+		gok_main_check_accessibility();
+	}
 	
 	if (!gok_main_check_sticky_keys (m_pWindowMain)) {
 		return GOK_ERROR_NOSTICKYKEYS;
 	}
-	
+
 	/* display the "main" keyboard */
 	if (gok_main_display_scan_main () == FALSE)
 	{
@@ -483,10 +501,10 @@ gok_log ("finished check");
 /* private */
 gboolean gok_main_display_scan_main()
 {
-	if (gok_main_display_scan ((GokKeyboard*)NULL, mainkeyboardname, KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL) == FALSE)
+	if (gok_main_display_scan ((GokKeyboard*)NULL, mainkeyboardname, KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL, KEYBOARD_SHAPE_BEST) == FALSE)
 	{
 		/* if no "main" keyboard then display first keyboard in list */
-		if (gok_main_display_scan ((GokKeyboard*)NULL, NULL, KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL) == FALSE)
+		if (gok_main_display_scan ((GokKeyboard*)NULL, NULL, KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL, KEYBOARD_SHAPE_BEST) == FALSE)
 		{
 			gok_main_display_error (_("No keyboards to display!"));
 			return FALSE;
@@ -628,8 +646,7 @@ void gok_main_window_change_listener (Accessible* pAccessible)
 		gok_scanner_start();
 	}
 
-	/* hack to make sure gok is not occluded - seems to be unstable*/
-	gdk_window_raise (m_pWindowMain->window);
+	gok_main_raise_window ();
 
 	gok_log_leave();	
 }
@@ -702,9 +719,8 @@ void gok_main_button_listener (gint button, gint state, long mods, long timestam
 
 	if (state == 0)
 	{
-		/* hack to make sure gok is not occluded - TODO: seems to be unstable*/
-		gdk_window_raise (m_pWindowMain->window);
-		
+		gok_main_raise_window ();
+
 		switch (button)
 		{
 			case 1:
@@ -755,6 +771,20 @@ void gok_main_button_listener (gint button, gint state, long mods, long timestam
 	gok_log_leave();	
 }
 
+/**
+ * gok_main_raise_window:
+ * hack to make sure gok is not occluded - TODO: seems to be unstable
+ *
+ **/
+void
+gok_main_raise_window (void)
+{
+	if (m_pWindowMain && m_pWindowMain->window) 
+	{
+		gdk_window_raise (m_pWindowMain->window);
+	}
+}
+
 /**
 * gok_main_mouse_button_listener
 * @button: Mouse button number that has changed state.
@@ -771,8 +801,7 @@ void gok_main_mousebutton_listener (gint button, gint state, long mods, long tim
 
 	if (state == 0)
 	{
-		/* hack to make sure gok is not occluded - TODO: seems to be unstable*/
-		gdk_window_raise (m_pWindowMain->window);
+		gok_main_raise_window ();
 
 		/* TODO: account for reversed button order, which is possible! 
 		 * i.e. leftbutton == 3, etc.
@@ -1031,9 +1060,10 @@ gok_main_keyboard_find_byname (const gchar *NameKeyboard)
 /**
 * gok_main_display_scan
 * @pKeyboard: If this is supplied it takes precedence over the name parameter (useful for dynamic keyboards with the same name)
-* @NameKeyboard: Name of the keyboard you want displayed (must be in the list of keyboards.)
-* @TypeKeyboard: Usually NORMAL but it can be used to specify a particular type of 
-* keyboard (Example: center-weighted or upper-left-weighted).
+* @nameKeyboard: Name of the keyboard you want displayed (must be in the list of keyboards.)
+# @typeKeyboard: Can be used to describe a runtime keyboard type (e.g. a Menus keyboard)
+* @layout: Can be used to specify a particular type of keyboard (Example: center-weighted or upper-left-weighted).
+* @shape: Can be used to specify a particular shape of keyboard (Example: square).
 *
 * Display a keyboard and allow the user to make selections from it.
 * The keyboard must have already been created and in the list of keyboards.
@@ -1043,8 +1073,11 @@ gok_main_keyboard_find_byname (const gchar *NameKeyboard)
 *
 * returns: TRUE if keyboard is displayed, FALSE if not.
 */
-gboolean gok_main_display_scan (GokKeyboard* pKeyboard, gchar* NameKeyboard, KeyboardTypes TypeKeyboard, KeyboardLayouts LayoutType)
+gboolean 
+gok_main_display_scan (GokKeyboard* pKeyboard, gchar* nameKeyboard, KeyboardTypes typeKeyboard, KeyboardLayouts layout, KeyboardShape shape)
 {
+	/* FIXME we're not using the type params */
+	
 	GokKeyboard* pKB;
 	gboolean bPushed;
 	gboolean bReturnCode;
@@ -1061,7 +1094,7 @@ gboolean gok_main_display_scan (GokKeyboard* pKeyboard, gchar* NameKeyboard, Key
 	}
 	else
 	{
-		pKB = gok_main_keyboard_find_byname (NameKeyboard);
+		pKB = gok_main_keyboard_find_byname (nameKeyboard);
 	}
 
 	if (pKB == NULL)
@@ -1113,8 +1146,7 @@ gboolean gok_main_ds (GokKeyboard* pKeyboard)
 	gok_scanner_stop();
 
 	/* raise the window as a precaution */
-	if (m_pWindowMain && m_pWindowMain->window)
-		gdk_window_raise (m_pWindowMain->window);
+	gok_main_raise_window ();
 
 	/* create new keys for dynamic keyboards */
 	if (pKeyboard->bDynamicallyCreated == TRUE)
@@ -1127,7 +1159,9 @@ gboolean gok_main_ds (GokKeyboard* pKeyboard)
 	}
 	
 	/* layout the keys on the keyboard */
-	if (gok_keyboard_layout (pKeyboard) == FALSE)
+	if (gok_keyboard_layout (pKeyboard, KEYBOARD_LAYOUT_NORMAL, 
+		 	/*KEYBOARD_SHAPE_WIDE KEYBOARD_SHAPE_SQUARE*/ 
+			KEYBOARD_SHAPE_KEYSQUARE, FALSE) == FALSE)
 	{
 		gok_log_leave();
 		return FALSE;
@@ -1335,6 +1369,7 @@ GokKeyboard* gok_main_read_keyboards_from_dir (const char *directory)
 	gchar* complete_path;
 
 	gok_log_enter();
+	gok_log ("directory: %s", directory);
 	pKeyboardFirst = NULL;
 	pKeyboard = NULL;
 	pKeyboardPrevious = NULL;
@@ -1352,7 +1387,11 @@ GokKeyboard* gok_main_read_keyboards_from_dir (const char *directory)
 	/* create the default "core" keyboard (aka qwerty keyboard) */
 	if (gok_data_get_use_xkb_kbd ()) {
 		pKeyboard = pKeyboardFirst = gok_keyboard_get_core ();
-		pKeyboard->pKeyboardPrevious = NULL;
+		/* pKeyboard = pKeyboardFirst = gok_keyboard_get_alpha_by_frequency (); *//* TESTING ONLY!!!! */
+		if (pKeyboard)
+			pKeyboard->pKeyboardPrevious = NULL;
+		else
+		        gok_data_set_use_xkb_kbd (FALSE);
 		pKeyboardPrevious = pKeyboard;
 	}
 
@@ -1463,7 +1502,9 @@ void gok_main_initialize_wordcomplete ()
     }
     else
     {
-	if ( !(gok_wordcomplete_open (directory_name)) ) {
+	if ( !(gok_wordcomplete_open (gok_wordcomplete_get_default (),
+				      directory_name)) ||
+		! (gok_keyslotter_on (gok_wordcomplete_get_default (), KEYTYPE_WORDCOMPLETE))) {
 	    gok_log_x ("Error initializing word completion");
 	}
 	
@@ -1828,7 +1869,7 @@ void gok_main_close()
 	gok_action_close();
 	gok_feedback_close();
 	gok_control_button_callback_close();
-	gok_wordcomplete_close();
+	gok_wordcomplete_close(gok_wordcomplete_get_default ());
 	gok_predictor_close(m_pCommandPredictor);
 	
 	_exit(0);
@@ -2032,17 +2073,19 @@ gok_main_check_sticky_keys (GtkWidget *widget)
 						      XkbStickyKeysMask)) {
 #endif
 				xkb_ok = TRUE;
-				dialog = gtk_message_dialog_new (GTK_WINDOW (widget),
+				if (!bLoginOption) {
+				  dialog = gtk_message_dialog_new (NULL,
 								 GTK_DIALOG_DESTROY_WITH_PARENT,
 								 GTK_MESSAGE_INFO,
 								 GTK_BUTTONS_OK,
 								 "GOK has enabled Sticky Keys, which it requires.\n");
 				
-				/* Destroy the dialog when the user responds to it (e.g. clicks a button) */
-				g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
+				  /* Destroy the dialog when the user responds to it (e.g. clicks a button) */
+				  g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
 							  G_CALLBACK (gtk_widget_destroy),
 							  GTK_OBJECT (dialog));
-				gtk_widget_show_all (dialog);
+				  gtk_widget_show_all (dialog);
+				}
 			}
 			else
 				xkb_ok = FALSE;
@@ -2050,16 +2093,18 @@ gok_main_check_sticky_keys (GtkWidget *widget)
 		XkbFreeKeyboard (xkb, 0, True);
 	}
 	if (xkb_ok == FALSE) {
+		if (!bLoginOption) {
                 /* post an error dialog */
-		dialog = gtk_message_dialog_new (GTK_WINDOW (widget),
+			dialog = gtk_message_dialog_new (NULL,
 						 GTK_DIALOG_DESTROY_WITH_PARENT,
 						 GTK_MESSAGE_ERROR,
 						 GTK_BUTTONS_CLOSE,
 						 "GOK cannot run because XKB display extension is missing.\n");		
-		g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
+			g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
 					  G_CALLBACK (gtk_widget_destroy),
 					  GTK_OBJECT (dialog));
-		gtk_widget_show_all (dialog);
+			gtk_widget_show_all (dialog);
+		}
 		return FALSE;
 	}
 	return TRUE;
@@ -2071,13 +2116,20 @@ static GtkWidget* acd = NULL; /* accessibility check dialog */
 
 void check_accessibility_cb ( GObject* o, gpointer* data )
 {
+	GnomeClient *client;
+
 	gtk_widget_destroy(acd);	
 	
 	if (strcmp((gchar*)data,"logout") == 0) {
 		gok_gconf_set_bool ( gok_data_get_gconf_client(),
 			GCONF_ACCESSIBILITY_KEY, TRUE );
-			gok_main_close();  /* FIXME: need to logout */
-			_exit (0);
+		if (!(client = gnome_master_client())) {
+			gok_main_close();
+		}
+ 		gnome_client_request_save (client, GNOME_SAVE_GLOBAL, TRUE,
+			GNOME_INTERACT_ANY, FALSE, TRUE); /* code borrowed from 
+			gnome-control-center/ capplets/ accessibility/ at-properties/ 
+			main.c */
 	}
 	else if (strcmp((gchar*)data,"quit") == 0) {
 		gok_main_close();
@@ -2106,9 +2158,9 @@ gok_main_check_accessibility ()
 		gboolean 	returnCode = TRUE;
 		
 		acd = gtk_message_dialog_new (
-			GTK_WINDOW(m_pWindowMain), 
+			NULL, 
 			GTK_DIALOG_DESTROY_WITH_PARENT,
-			GTK_MESSAGE_WARNING,
+			GTK_MESSAGE_INFO, /* FIXME: GTK_MESSAGE_WARNING occludes keyboard */
 			GTK_BUTTONS_NONE,			
 			_("Assistive Technology Support Is Not Enabled." 
 		"\n\n" 
